package net.wanji.utc.dt.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.RequiredArgsConstructor;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dao.entity.CrossSchedulesPO;
import net.wanji.databus.dto.PlanSectionDTO;
import net.wanji.databus.dto.SchemePhaseLightsDTO;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import net.wanji.utc.dt.service.StaticInfoService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.ws.rs.core.MediaType;
import java.util.Date;
import java.util.List;

/**
 * 静态信息接口
 *
 * @date 2022/11/15 9:38
 */
@Api(value = "静态信息接口", tags = "静态信息接口")
@RequestMapping("/staticInfo")
@RestController
@RequiredArgsConstructor
public class StaticInfoController {

    private final StaticInfoService staticInfoService;

    @AspectLog(description = "方案数据-方案信息、相位信息、灯组信息", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/schemePhaseLights",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "方案数据-方案信息、相位信息、灯组信息", notes = "方案数据-方案信息、相位信息、灯组信息",
            response = SchemePhaseLightsVO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemePhaseLightsVO.class)
    })
    public JsonViewObject schemePhaseLights(@RequestBody @Validated SchemePhaseLightsDTO schemePhaseLightsDTO) throws Exception {
        SchemePhaseLightsVO schemePhaseLightsVO = staticInfoService.schemePhaseLights(schemePhaseLightsDTO);
        return JsonViewObject.newInstance().success(schemePhaseLightsVO);
    }

    @AspectLog(description = "计划数据-计划信息、时段信息", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/planSection",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "计划数据-计划信息、时段信息", notes = "计划数据-计划信息、时段信息", response = PlanSectionVO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = PlanSectionVO.class)
    })
    public JsonViewObject planSection(@RequestBody @Validated PlanSectionDTO planSectionDTO) throws Exception {
        List<PlanSectionVO> planSectionVOS = staticInfoService.planSection(planSectionDTO);
        return JsonViewObject.newInstance().success(planSectionVOS);
    }

    @AspectLog(description = "时间表数据", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/crossSchedules", produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossSchedulesPO.class)
    })
    public JsonViewObject crossSchedules(@RequestParam String crossId) throws Exception {
        List<CrossSchedulesPO> crossSchedulesPOS = staticInfoService.crossSchedules(crossId);
        return JsonViewObject.newInstance().success(crossSchedulesPOS);
    }

    @AspectLog(description = "查询信号机时间", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/signalTime", produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossSchedulesPO.class)
    })
    public JsonViewObject signalTime(@RequestParam String crossId) throws Exception {
        Date signalTime = staticInfoService.signalTime(crossId);
        return JsonViewObject.newInstance().success(signalTime);
    }
}
