package net.wanji.utc.dt.netty.response.impl;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.dt.cache.CrossInfoCache;
import net.wanji.utc.dt.common.constants.Constants;
import net.wanji.utc.dt.netty.NettyClient;
import net.wanji.utc.dt.netty.pojo.CommandPojo;
import net.wanji.utc.dt.netty.response.CommandResponseFactory;
import net.wanji.utc.dt.pojo.netty.MessageResultPojo;
import net.wanji.databus.po.CrossInfoPO;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * @author duanruiming
 * @date 2023/05/12 11:00
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class HeartBeatService implements CommandResponseFactory {

    private final ThreadPoolTaskExecutor threadPoolTaskExecutor;

    @Override
    public String getCommandResponse(CommandPojo commandPojo) {
        return commandPojo.getResponseMsg();
    }

    @Scheduled(fixedRate = 30 * 1000, initialDelay = 10 * 1000)
    public void heartbeat() {
        Map<String, CrossInfoPO> crossInfoMap = CrossInfoCache.getCrossInfoCache();
        for (Map.Entry<String, CrossInfoPO> entry : crossInfoMap.entrySet()) {
            CrossInfoPO crossInfoPO = entry.getValue();
            String ip = crossInfoPO.getIp();
            Integer port = crossInfoPO.getPort();
            threadPoolTaskExecutor.execute(() -> {
                String heartBeatRequest = Constants.buildMessage(Constants.COMMAND_HEART_BEAT_QUERY);
                MessageResultPojo resultPojo = NettyClient.sendMessage(ip, port, heartBeatRequest, Constants.COMMAND_HEART_BEAT_REPLY, 300);
            });
        }
    }
}
