package net.wanji.utc.dt.service.impl;

import cn.hutool.core.util.HexUtil;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.dt.cache.SignalDataCache;
import net.wanji.utc.dt.common.constants.Constants;
import net.wanji.utc.dt.common.enums.ControlModelEnum;
import net.wanji.utc.dt.common.enums.ControlTypeEnum;
import net.wanji.utc.dt.common.enums.PhaseLightColorEnum;
import net.wanji.utc.dt.netty.NettyClient;
import net.wanji.utc.dt.pojo.dtconvert.*;
import net.wanji.utc.dt.pojo.netty.MessageResultPojo;
import net.wanji.utc.dt.service.SignalStatusService;
import net.wanji.utc.dt.util.CommonUtils;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 17:28
 */
@Service
@SuppressWarnings("all")
public class DTSignalStatusServiceImpl implements SignalStatusService {

    @Override
    public List<SignalStatusLogPO> runningStatusAlarm(String crossId) throws Exception {
        List<String> lightsStatus = SignalDataCache.realTimeLightCache.get(crossId);
        List<FaultInfoPojo> faultInfoPojos = getFaultInfoPojos(crossId);
        List<RunningStatusPojo> runningStatusPojos = SignalDataCache.realTimeRunStatusCache.get(crossId);
        return getSignalStatus(crossId, faultInfoPojos, runningStatusPojos);
    }

    private List<SignalStatusLogPO> getSignalStatus(String crossId, List<FaultInfoPojo> faultInfoPojos, List<RunningStatusPojo> runningStatusPojos) {
        List<SignalStatusLogPO> signalStatusLogPOS = new ArrayList<>();
        for (RunningStatusPojo runningStatusPojo : runningStatusPojos) {
            SignalStatusLogPO signalStatusLogPO = new SignalStatusLogPO();
            signalStatusLogPO.setCrossId(crossId);
            signalStatusLogPO.setSignalId(crossId);
            signalStatusLogPO.setStatus(1);
            if (CollectionUtils.isEmpty(faultInfoPojos)) {
                signalStatusLogPO.setFaultType(0);
            } else {
                signalStatusLogPO.setFaultType(1);
            }
            signalStatusLogPO.setControlType(ControlTypeEnum.getWjControlType(runningStatusPojo.getControlMode()));
            signalStatusLogPOS.add(signalStatusLogPO);
        }

        return signalStatusLogPOS;
    }

    public static List<RunningStatusPojo> getRunningStatusPojos(String crossId) throws Exception {
        List<RunningStatusPojo> runningStatusPojos = new ArrayList<>();
        CrossInfoPO crossInfoPO = DTControlCommandServiceImpl.checkCrossId(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_RUNNING_STATUS);
        String runningStatusSign = Constants.COMMAND_RUNNING_STATUS.concat(Constants.COMMAND_QUERY_REPLY);
        String runnningStatusHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), runnningStatusHex, runningStatusSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int size = body.length() / 14;
            List<String> paramList = new ArrayList<>();
            for (int i = 0; i < size; i++) {
                String param = body.substring(i * 14 + 6, i * 14 + 10);
                if (!StringUtils.equalsIgnoreCase("0000", param)) {
                    paramList.add(param);
                }
            }
            for (int i = 0; i < paramList.size() / 4; i++) {
                RunningStatusPojo runningStatusPojo = new RunningStatusPojo();
                runningStatusPojo.setControlCrossId(HexUtil.hexToInt(paramList.get(i * 4)));
                runningStatusPojo.setControlMode(paramList.get(i * 4 + 1));
                runningStatusPojo.setSchemeId(HexUtil.hexToInt(paramList.get(i * 4 + 2)));
                runningStatusPojo.setPhaseStageId(HexUtil.hexToInt(paramList.get(i * 4 + 3)));
                runningStatusPojos.add(runningStatusPojo);
            }
        }
        return runningStatusPojos;
    }

    public static List<String> getLightsStatusInfoPojos(String crossId) throws Exception {
        List<String> lightsStatusList = new ArrayList<>();
        CrossInfoPO crossInfoPO = DTControlCommandServiceImpl.checkCrossId(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_LIGHT_STATUS);
        String lightStatusSign = Constants.COMMAND_LIGHT_STATUS.concat(Constants.COMMAND_QUERY_REPLY);
        String lightStatusHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), lightStatusHex, lightStatusSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int lightsStatusSize = body.length() / 14;
            for (int i = 0; i < lightsStatusSize; i++) {
                String param = body.substring(i * 14 + 8, i * 14 + 10);
                if (!StringUtils.equalsIgnoreCase("00", param)) {
                    lightsStatusList.add(param);
                }
            }
        }
        return lightsStatusList;
    }

    private List<FaultInfoPojo> getFaultInfoPojos(String crossId) throws Exception {
        CrossInfoPO crossInfoPO = DTControlCommandServiceImpl.checkCrossId(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_FAULT_LIST);
        String faultSign = Constants.COMMAND_FAULT_LIST.concat(Constants.COMMAND_QUERY_REPLY);
        String faultHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), faultHex, faultSign, 300);
        List<FaultInfoPojo> faultInfoPojos = new ArrayList<>();
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int lightsStatusSize = body.length() / 14 * 64;
            int paramIndex = 8;
            for (int i = 0; i < lightsStatusSize; i++) {
                String param = body.substring(paramIndex * i, paramIndex * i + 8);


            }

        }
        return null;
    }

    @Override
    public synchronized List<LightsStatusVO> lightStatus(String crossId) throws Exception {
        List<String> lightsStatus = SignalDataCache.realTimeLightCache.get(crossId);
        List<PhaseStageStatusPojo> phaseStageStatusPojos = SignalDataCache.realTimePhaseStatusStageCache.get(crossId);
        List<RunningStatusPojo> runningStatusPojos = SignalDataCache.realTimeRunStatusCache.get(crossId);
        if (CollectionUtils.isEmpty(lightsStatus) || CollectionUtils.isEmpty(phaseStageStatusPojos) || CollectionUtils.isEmpty(runningStatusPojos)) {
            throw new Exception("获取东土信号机实时数据为空");
        }
        return getLightsStatusVOs(crossId, lightsStatus, phaseStageStatusPojos, runningStatusPojos);
    }

    public static List<PhaseStageStatusPojo> getPhaseStageStatus(String crossId) throws Exception {
        List<PhaseStageStatusPojo> phaseStageStatusPojos = new ArrayList<>();
        CrossInfoPO crossInfoPO = DTControlCommandServiceImpl.checkCrossId(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_PHASE_STAGE_STATUS);
        String stageStatusSign = Constants.COMMAND_PHASE_STAGE_STATUS.concat(Constants.COMMAND_QUERY_REPLY);
        String stageStatusHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), stageStatusHex, stageStatusSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int size = body.length() / 46;
            String matchStr = "0603%s%s";
            int index = 0;
            for (int i = 0; i < size; i++) {
                PhaseStageStatusPojo phaseStageStatusPojo = new PhaseStageStatusPojo();
                for (int j = 2; j < 5; j++) {
                    String currentMatchStr = String.format(matchStr, String.format("%02x", j), String.format("%02x", i + 1));
                    index = body.indexOf(currentMatchStr) + 8;
                    if (j == 2) {
                        String stageStatus = body.substring(index, index + 2);
                        phaseStageStatusPojo.setPhaseStageStatus(stageStatus);
                    }
                    if (j == 3) {
                        phaseStageStatusPojo.setRunningTime(HexUtil.hexToInt(body.substring(index, index + 4)));
                    }
                    if (j == 4) {
                        phaseStageStatusPojo.setRemainTime(HexUtil.hexToInt(body.substring(index, index + 4)));
                    }
                }
                phaseStageStatusPojo.setPhaseStageId(i + 1);
                phaseStageStatusPojos.add(phaseStageStatusPojo);
            }
        }
        return phaseStageStatusPojos.stream().filter(phaseStageStatusPojo -> !StringUtils.equals(phaseStageStatusPojo.getPhaseStageStatus(), "10")).collect(Collectors.toList());
    }

    private List<LightsStatusVO> getLightsStatusVOs(String crossId, List<String> lightsStatus, List<PhaseStageStatusPojo> phaseStageStatusPojos, List<RunningStatusPojo> runningStatusPojos) {
        List<LightsStatusVO> lightsStatusVOS = new ArrayList<>(runningStatusPojos.size());
        for (int j = 0; j < runningStatusPojos.size(); j++) {
            RunningStatusPojo runningStatusPojo = runningStatusPojos.get(j);
            LightsStatusVO lightsStatusVO = new LightsStatusVO();
            Map<String, Object> phaseMap = new HashMap<>();
            String controlModel = String.valueOf(ControlModelEnum.getWjControlMode(runningStatusPojo.getControlMode().substring(2, 4)));
            lightsStatusVO.setCrossId(crossId);
            lightsStatusVO.setRunMode(controlModel);
            lightsStatusVO.setControlMode(controlModel);
            PhaseStageStatusPojo phaseStageStatusPojo = new PhaseStageStatusPojo();
            if (!CollectionUtils.isEmpty(phaseStageStatusPojos)) {
                phaseStageStatusPojo = phaseStageStatusPojos.get(0);
            }

            Integer currentSchemeId = runningStatusPojo.getSchemeId();
            Integer currentPhaseStageId = runningStatusPojo.getPhaseStageId();
            List<SchemeInfoPojo> schemeInfoPojos = SignalDataCache.schemeInfoCache.get(crossId);

            if (!CollectionUtils.isEmpty(schemeInfoPojos)) {
                for (SchemeInfoPojo schemeInfoPojo : schemeInfoPojos) {
                    if (Objects.equals(schemeInfoPojo.getSchemeId(), currentSchemeId)) {
                        List<Integer> phaseStageTimeChain = schemeInfoPojo.getPhaseStageTimeChain();
                        List<Integer> phaseStageChain = schemeInfoPojo.getPhaseStageChain();
                        List<PhaseStageInfoPojo> phaseStageInfoPojos = SignalDataCache.phaseStageInfoCache.get(crossId);
                        if (!CollectionUtils.isEmpty(phaseStageInfoPojos)) {
                            for (int l = 0; l < phaseStageInfoPojos.size(); l++) {
                                PhaseStageInfoPojo phaseStageInfoPojo = phaseStageInfoPojos.get(l);
                                if (Objects.equals(phaseStageInfoPojo.getPhaseStageId(), currentPhaseStageId)) {
                                    // 通过阶段好号取阶段时间
                                    Integer phaseStageTime = 0;
                                    for (int i = 0; i < phaseStageChain.size(); i++) {
                                        Integer phaseStage = phaseStageChain.get(i);
                                        if (Objects.equals(currentPhaseStageId, phaseStage)) {
                                            phaseStageTime = phaseStageTimeChain.get(i);
                                        }
                                    }
                                    // 通过相位号获取过渡时间
                                    List<Integer> phaseList = phaseStageInfoPojo.getPhaseList();
                                    List<PhaseInfoPojo> phaseInfoPojos = SignalDataCache.phaseInfoCache.get(crossId);
                                    Integer redTime = 0;
                                    for (PhaseInfoPojo phaseInfoPojo : phaseInfoPojos) {
                                        Integer lightsId = phaseInfoPojo.getLightsId();
                                        if (phaseInfoPojo.getPhaseId() == phaseList.get(0)) {
                                            redTime = phaseInfoPojo.getRedTime();
                                        }
                                        List<LightsInfoPojo> lightsInfoPojos = SignalDataCache.lightInfoCache.get(crossId);
                                        for (LightsInfoPojo lightsInfoPojo : lightsInfoPojos) {
                                            if (Objects.equals(lightsInfoPojo.getLightId(), lightsId)) {
                                                Integer lightsDir = lightsInfoPojo.getLightsDir(lightsId);
                                                Integer type = lightsInfoPojo.getType();
                                                Integer lightsTurn = lightsInfoPojo.getTurnDir(lightsDir, type);
                                                phaseMap.put(String.join("-", String.valueOf(lightsDir), String.valueOf(lightsTurn)), phaseStageTime);
                                            }
                                        }
                                    }
                                    lightsStatusVO.setSchemeId(String.valueOf(currentSchemeId));
                                    lightsStatusVO.setCycleLen(schemeInfoPojo.getCycle());
                                    lightsStatusVO.setCycleCountDown(phaseStageStatusPojo.getRemainTime());
                                    lightsStatusVO.setPhaseMap(phaseMap);
                                    // 当运行时间小于过渡时，运行的时上一个阶段，
                                    int realRemainTime = phaseStageStatusPojo.getRemainTime();
                                    Integer runningTime = phaseStageStatusPojo.getRunningTime();
                                    if (realRemainTime <= phaseStageTime - redTime) {
                                        lightsStatusVO.setPhaseId(String.valueOf(currentPhaseStageId));
                                        lightsStatusVO.setCyclePhaseCountDown(realRemainTime);
                                    } else {
                                        Integer preStageId = null;
                                        int index = phaseStageChain.indexOf(currentPhaseStageId);
                                        if (index == 0) {
                                            preStageId = phaseStageChain.get(phaseStageChain.size() - 1);
                                        } else {
                                            preStageId = phaseStageChain.get(index - 1);
                                        }
                                        lightsStatusVO.setPhaseId(String.valueOf(preStageId));
                                        lightsStatusVO.setCyclePhaseCountDown(phaseStageStatusPojo.getRemainTime());
                                    }
                                }
                            }
                        }
                        lightsStatusVO.setDirLampGroupMap(getDirLampGroupMap(lightsStatus));
                    }
                }
            }
            lightsStatusVOS.add(lightsStatusVO);
        }
        return lightsStatusVOS;
    }

    @NotNull
    private static Map<String, Object> getDirLampGroupMap(List<String> lightsStatus) {
        Map<String, Object> dirLampGroupMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(lightsStatus)) {
            for (int i = 0; i < lightsStatus.size(); i++) {
                Map<Integer, String> turnColorMap = new HashMap<>();
                String wjLightColor = PhaseLightColorEnum.getWjLightColor(lightsStatus.get(i));
                Integer turn = CommonUtils.getTurnByLightsId(i);
                turnColorMap.put(turn, wjLightColor);
                String dir = String.valueOf(CommonUtils.getDirByLightsId(i));
                if (dirLampGroupMap.isEmpty() || Objects.isNull(dirLampGroupMap.get(dir))) {
                    dirLampGroupMap.put(dir, turnColorMap);
                } else {
                    Map<Integer, String> currentTurnColorMap = (Map<Integer, String>) dirLampGroupMap.get(dir);
                    currentTurnColorMap.put(turn, wjLightColor);
                    dirLampGroupMap.put(dir, currentTurnColorMap);
                }
            }
        }
        return dirLampGroupMap;
    }
}
