package net.wanji.utc.dt.service.impl;

import cn.hutool.core.util.HexUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.DateStyle;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dto.PlanSectionDTO;
import net.wanji.databus.dto.SchemePhaseLightsDTO;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import net.wanji.utc.dt.cache.SignalDataCache;
import net.wanji.utc.dt.common.constants.Constants;
import net.wanji.utc.dt.common.enums.ControlModelEnum;
import net.wanji.utc.dt.netty.NettyClient;
import net.wanji.utc.dt.pojo.dtconvert.*;
import net.wanji.utc.dt.pojo.netty.MessageResultPojo;
import net.wanji.utc.dt.service.StaticInfoService;
import net.wanji.utc.dt.util.CommonUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 17:29
 */
@Service
@RequiredArgsConstructor
@Slf4j
@SuppressWarnings("all")
public class DTStaticInfoServiceImpl implements StaticInfoService {

    private final SignalDataCache signalDataCache;

    @Override
    public SchemePhaseLightsVO schemePhaseLights(SchemePhaseLightsDTO schemePhaseLightsDTO) throws Exception {
        String crossId = schemePhaseLightsDTO.getCrossId();
        CrossInfoPO crossInfo = DTControlCommandServiceImpl.checkCrossId(crossId);
        List<PhaseInfoPojo> phaseInfoPojos = getPhaseInfo(crossInfo);
        List<LightsInfoPojo> lightsInfoPojos = getLightsInfo(crossInfo);
        List<PhaseStageInfoPojo> phaseStageInfoPojos = getPhaseStageInfo(crossInfo);
        List<SchemeInfoPojo> schemeInfoPojos = getSchemeInfo(crossInfo);
        SchemePhaseLightsVO schemePhaseLightsVO = getSchemePhaseLightsVO(crossId, phaseInfoPojos, lightsInfoPojos, phaseStageInfoPojos, schemeInfoPojos);
        return schemePhaseLightsVO;
    }

    private SchemePhaseLightsVO getSchemePhaseLightsVO(String crossId, List<PhaseInfoPojo> phaseInfoPojos, List<LightsInfoPojo> lightsInfoPojos, List<PhaseStageInfoPojo> phaseStageInfoPojos, List<SchemeInfoPojo> schemeInfoPojos) {
        SchemePhaseLightsVO schemePhaseLightsVO = new SchemePhaseLightsVO();
        List<CrossSchemePO> crossSchemeList = getCrossSchemeList(crossId, schemeInfoPojos);
        List<CrossPhasePO> crossPhaseList = getCrossPhaseList(crossId, phaseInfoPojos, phaseStageInfoPojos, schemeInfoPojos);
        List<CrossLightsPO> crossLightsList = getCrossLightsList(crossId, phaseInfoPojos, lightsInfoPojos);
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = getCrossPhaseLightsList(crossId, crossPhaseList, phaseInfoPojos, phaseStageInfoPojos);
        schemePhaseLightsVO.setCrossPhaseLightsPOList(crossPhaseLightsPOList);
        schemePhaseLightsVO.setCrossPhaseList(crossPhaseList);
        schemePhaseLightsVO.setCrossLightsList(crossLightsList);
        schemePhaseLightsVO.setCrossSchemeList(crossSchemeList);
        return schemePhaseLightsVO;
    }

    private List<CrossPhaseLightsPO> getCrossPhaseLightsList(String crossId, List<CrossPhasePO> crossPhaseList, List<PhaseInfoPojo> phaseInfoPojos, List<PhaseStageInfoPojo> phaseStageInfoPojos) {
        List<CrossPhaseLightsPO> crossPhaseLightsList = new ArrayList<>(crossPhaseList.size());
        for (CrossPhasePO crossPhasePO : crossPhaseList) { // 当前相位号为阶段号
            String phaseNo = crossPhasePO.getPhaseNo();
            List<Integer> phaseList = new ArrayList<>();
            for (PhaseStageInfoPojo phaseStageInfoPojo : phaseStageInfoPojos) {
                if (StringUtils.equals(phaseNo, String.valueOf(phaseStageInfoPojo.getPhaseStageId()))) {
                    phaseList = phaseStageInfoPojo.getPhaseList();
                }
            }
            for (PhaseInfoPojo phaseInfoPojo : phaseInfoPojos) {
                if (phaseList.contains(phaseInfoPojo.getPhaseId())) {
                    CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
                    crossPhaseLightsPO.setPhaseId(Integer.valueOf(phaseNo));
                    crossPhaseLightsPO.setLightsId(phaseInfoPojo.getLightsId());
                    crossPhaseLightsPO.setCrossId(crossId);
                    crossPhaseLightsList.add(crossPhaseLightsPO);
                }
            }
        }
        return crossPhaseLightsList.stream().distinct().collect(Collectors.toList());
    }

    private List<CrossLightsPO> getCrossLightsList(String crossId, List<PhaseInfoPojo> phaseInfoPojos, List<LightsInfoPojo> lightsInfoPojos) {
        List<CrossLightsPO> crossLightsList = new ArrayList<>();
        for (PhaseInfoPojo phaseInfoPojo : phaseInfoPojos) {
            CrossLightsPO crossLightsPO = new CrossLightsPO();
            Integer phaseLightsId = phaseInfoPojo.getLightsId();
            for (LightsInfoPojo lightsInfoPojo : lightsInfoPojos) {
                Integer lightId = lightsInfoPojo.getLightId();
                if (Objects.equals(phaseLightsId, lightId)) {
                    crossLightsPO.setLightsNo(String.valueOf(lightId));
                    crossLightsPO.setName(StringUtils.join("灯组", lightId));
//                    Integer turnDir = LightsInfoPojo.getTurnDir(lightId, lightsInfoPojo.getType());
                    Integer turnDir = CommonUtils.getTurnByLightsId(lightId - 1);
                    Integer dir = CommonUtils.getDirByLightsId(lightId - 1);
                    crossLightsPO.setTurn(turnDir);
                    crossLightsPO.setType(2);
                    if (turnDir == 20) {
                        crossLightsPO.setType(3);
                    }
                    crossLightsPO.setCrossId(crossId);
                    crossLightsPO.setDir(dir);
                    crossLightsPO.setInOutType(2);
                    crossLightsList.add(crossLightsPO);
                    break;
                }
            }
        }
        return crossLightsList.stream().distinct().collect(Collectors.toList());
    }

    /**
     * 转化为万集单环模式，阶段号就是相位号
     *
     * @param crossId
     * @param phaseInfoPojos
     * @param phaseStageInfoPojos
     * @param schemeInfoPojos
     * @return
     */
    private List<CrossPhasePO> getCrossPhaseList(String crossId, List<PhaseInfoPojo> phaseInfoPojos, List<PhaseStageInfoPojo> phaseStageInfoPojos, List<SchemeInfoPojo> schemeInfoPojos) {
        List<CrossPhasePO> crossPhaseList = new ArrayList<>();
        for (PhaseStageInfoPojo phaseStageInfoPojo : phaseStageInfoPojos) {
            int sort = 0;
            Integer phaseStageId = phaseStageInfoPojo.getPhaseStageId();
            Integer schemeId = 0;
            Integer phaseStageTime = 0;
            for (SchemeInfoPojo schemeInfoPojo : schemeInfoPojos) {
                List<SchemeInfoPojo.SchemeStagePojo> stageInfoList = SchemeInfoPojo.getStageInfoList(schemeInfoPojo);
                for (SchemeInfoPojo.SchemeStagePojo schemeStagePojo : stageInfoList) {
                    if (Objects.equals(phaseStageId, schemeStagePojo.getPhaseStageId())) {
                        sort = stageInfoList.indexOf(schemeStagePojo) + 1;
                        schemeId = schemeStagePojo.getSchemeId();
                        phaseStageTime = schemeStagePojo.getPhaseStageTime();
                    }
                }
            }
            List<Integer> phaseList = phaseStageInfoPojo.getPhaseList();
            CrossPhasePO crossPhasePO = new CrossPhasePO();
            for (Integer phaseId : phaseList) {
                crossPhasePO.setCrossId(crossId);
                crossPhasePO.setPhaseNo(String.valueOf(phaseStageId));
                crossPhasePO.setName(StringUtils.join("相位", phaseStageId));
                crossPhasePO.setSort(sort);
                crossPhasePO.setPlanId(schemeId);
                crossPhasePO.setRingNo(1);
                crossPhasePO.setGroupNo(1);
                crossPhasePO.setControlMode(1);
                crossPhasePO.setPhaseType(1);
                PhaseInfoPojo phaseInfoPojo = PhaseInfoPojo.getPhaseInfoPojoByPhaseId(phaseId, phaseInfoPojos);
                if (Objects.nonNull(phaseInfoPojo)) {
                    int redTime = phaseInfoPojo.getRedTime() - phaseInfoPojo.getGreenFlashTime() - phaseInfoPojo.getGreenTime();
                    if (phaseStageTime == 0) { // 相位阶段没有对应方案
                        phaseStageTime = phaseInfoPojo.getGreenTime() + phaseInfoPojo.getGreenFlashTime() + phaseInfoPojo.getYellowTime() + redTime;
                    }
                    crossPhasePO.setPhaseTime(phaseStageTime);
                    int greenTime = phaseStageTime - phaseInfoPojo.getYellowTime() - redTime - phaseInfoPojo.getGreenFlashTime() * 2;
                    if (greenTime < 0) {
                        greenTime = phaseInfoPojo.getGreenTime();
                    }
                    crossPhasePO.setGreenTime(greenTime);
                    crossPhasePO.setGreenFlashTime(phaseInfoPojo.getGreenFlashTime());
                    crossPhasePO.setYellowFlashTime(phaseInfoPojo.getYellowFlashTime() * 2);
                    crossPhasePO.setYellowTime(phaseInfoPojo.getYellowTime());
                    crossPhasePO.setRedTime(redTime);
                    crossPhasePO.setMinGreenTime(phaseInfoPojo.getMinGreenTime());
                    crossPhasePO.setMaxGreenTime(phaseInfoPojo.getMaxGreenTime());
                }
                crossPhaseList.add(crossPhasePO);
            }
        }
        return crossPhaseList.stream().distinct().collect(Collectors.toList());
    }

    private List<CrossSchemePO> getCrossSchemeList(String crossId, List<SchemeInfoPojo> schemeInfoPojos) {
        List<CrossSchemePO> crossSchemeList = new ArrayList<>();
        for (SchemeInfoPojo schemeInfoPojo : schemeInfoPojos) {
            CrossSchemePO crossSchemePO = new CrossSchemePO();
            crossSchemePO.setSchemeNo(String.valueOf(schemeInfoPojo.getSchemeId()));
            crossSchemePO.setName(StringUtils.join("方案", schemeInfoPojo.getSchemeId()));
            crossSchemePO.setCrossId(crossId);
            crossSchemePO.setCycle(schemeInfoPojo.getCycle());
            crossSchemePO.setCoordPhase(schemeInfoPojo.getHarmonizeId());
            crossSchemePO.setOffset(schemeInfoPojo.getOffset());
            crossSchemePO.setSource(1);
            crossSchemePO.setIsDeleted(0);
            crossSchemeList.add(crossSchemePO);
        }
        return crossSchemeList;
    }

    private List<SchemeInfoPojo> getSchemeInfo(CrossInfoPO crossInfo) {
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_SCHME_CONFIG);
        String schemeSign = Constants.COMMAND_SCHME_CONFIG.concat(Constants.COMMAND_QUERY_REPLY);
        String schemeHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfo.getIp(), crossInfo.getPort(), schemeHex, schemeSign, 300);
        return getSchemeInfoPojos(crossInfo, resultPojo);
    }

    private List<SchemeInfoPojo> getSchemeInfoPojos(CrossInfoPO crossInfo, MessageResultPojo resultPojo) {
        List<SchemeInfoPojo> schemeInfoPojos = new ArrayList<>();
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            Map<Integer, List<String>> schemeIdParamMap = getSchemeIdParamMap(hexMessageResult);
            schemeInfoPojos = paramMapConvert(schemeIdParamMap);
        }
        SignalDataCache.schemeInfoCache.put(crossInfo.getId(), schemeInfoPojos);
        return schemeInfoPojos;
    }

    private List<SchemeInfoPojo> paramMapConvert(Map<Integer, List<String>> schemeIdParamMap) {
        List<SchemeInfoPojo> schemeInfoPojos = new ArrayList<>();
        for (Map.Entry<Integer, List<String>> entry : schemeIdParamMap.entrySet()) {
            SchemeInfoPojo schemeInfoPojo = new SchemeInfoPojo();
            Integer schemeId = entry.getKey();
            schemeInfoPojo.setSchemeId(schemeId);
            List<String> value = entry.getValue();
            schemeInfoPojo.setControlCrossId(HexUtil.hexToInt(value.get(0)));
            schemeInfoPojo.setCycle(HexUtil.hexToInt(value.get(1)));
            schemeInfoPojo.setHarmonizeId(HexUtil.hexToInt(value.get(2)));
            schemeInfoPojo.setOffset(HexUtil.hexToInt(value.get(3)));
            String phaseChainHex = value.get(4);
            List<Integer> phaseList = getListFromHex(phaseChainHex, 2, "00");
            schemeInfoPojo.setPhaseStageChain(phaseList);

            String phaseTimeChainHex = value.get(5);
            List<Integer> phaseTimeList = getListFromHex(phaseTimeChainHex, 4, "0000");
            schemeInfoPojo.setPhaseStageTimeChain(phaseTimeList);

            String phaseTypeChainHex = value.get(6);
            List<Integer> phaseTypeList = getListFromHex(phaseTypeChainHex, 2, "00");
            schemeInfoPojo.setPhaseStageTypeChain(phaseTypeList);

            schemeInfoPojos.add(schemeInfoPojo);
        }
        return schemeInfoPojos.stream().filter(schemeInfoPojo -> schemeInfoPojo.getCycle() != 0).collect(Collectors.toList());
    }

    /**
     * 通用处理字符串多字节转int，固定长度
     * 00000000000000000000000008070605 ==>  5 6 7 8
     *
     * @param hex
     * @param offset
     * @param ignoreStr
     * @return
     */
    public static List<Integer> getListFromHex(String hex, int offset, String ignoreStr) {
        List<Integer> list = new ArrayList<>();
        for (int i = 0; i < hex.length() / offset; i++) {
            String phaseTypeHex = hex.substring(i * offset, i * offset + offset);
            if (!StringUtils.equals(ignoreStr, phaseTypeHex)) {
                list.add(HexUtil.hexToInt(phaseTypeHex));
            }
        }
        Collections.reverse(list);
        return list;
    }

    private static Map<Integer, List<String>> getSchemeIdParamMap(String hexMessageResult) {
        Map<Integer, List<String>> schemeIdParamMap = new HashMap<>();
        if (StringUtils.isNotBlank(hexMessageResult)) {
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int schemeSize = body.length() / 228;
            String matchStr = "0902%s%s";
            int schemeId = 0;
            for (int i = 0; i < schemeSize; i++) {
                ++schemeId;
                List<String> paramList = new ArrayList<>();
                // 属性ID数量
                for (int j = 2; j < 9; j++) {
                    String currentMatchStr = String.format(matchStr, String.format("%02x", j), String.format("%02x", schemeId));
                    int paramStartIndex = body.indexOf(currentMatchStr) + 8;
                    String param = null;
                    if (j == 2 || j == 4) {
                        param = body.substring(paramStartIndex, paramStartIndex + 2);
                    } else if (j == 3) {
                        param = body.substring(paramStartIndex, paramStartIndex + 8);
                    } else if (j == 5) {
                        param = body.substring(paramStartIndex, paramStartIndex + 4);
                    } else if (j == 6 || j == 8) {
                        param = body.substring(paramStartIndex, paramStartIndex + 32);
                    } else if (j == 7) {
                        param = body.substring(paramStartIndex, paramStartIndex + 64);
                    }
                    paramList.add(param);
                }
                schemeIdParamMap.put(schemeId, paramList);
            }
        }
        return schemeIdParamMap;
    }

    private List<PhaseStageInfoPojo> getPhaseStageInfo(CrossInfoPO crossInfo) {
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_PHASE_STAGE_CONFIG);
        String phaseStageSign = Constants.COMMAND_PHASE_STAGE_CONFIG.concat(Constants.COMMAND_QUERY_REPLY);
        String phaseStageHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfo.getIp(), crossInfo.getPort(), phaseStageHex, phaseStageSign, 300);
        return getPhaseStageInfoPojos(crossInfo, resultPojo);
    }

    private List<PhaseStageInfoPojo> getPhaseStageInfoPojos(CrossInfoPO crossInfo, MessageResultPojo resultPojo) {
        List<PhaseStageInfoPojo> phaseStageInfoPojos = new ArrayList<>();
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int phaseStageSize = body.length() / 308;
            for (int i = 0; i < phaseStageSize; i++) {
                String phaseStageHex = body.substring(i * 308 + 8, i * 308 + 24);
                StringBuffer sb = new StringBuffer();
                for (int j = 0; j < phaseStageHex.length() / 2; j++) {
                    String hex = phaseStageHex.substring(j * 2, j * 2 + 2);
                    String binary = Integer.toBinaryString(HexUtil.hexToInt(hex));
                    sb.append(StringUtils.leftPad(binary, 8, '0'));
                }
                List<Integer> phaseList = new ArrayList<>();
                char[] chars = sb.reverse().toString().toCharArray();
                for (int k = 0; k < chars.length; k++) {
                    if (chars[k] == '1') {
                        phaseList.add(k + 1);
                    }
                }
                if (CollectionUtils.isEmpty(phaseList)) {
                    continue;
                }
                PhaseStageInfoPojo phaseStageInfoPojo = new PhaseStageInfoPojo();
                phaseStageInfoPojo.setPhaseStageId(i + 1);
                phaseStageInfoPojo.setPhaseList(phaseList);
                phaseStageInfoPojos.add(phaseStageInfoPojo);
            }
        }
        SignalDataCache.phaseStageInfoCache.put(crossInfo.getId(), phaseStageInfoPojos);
        return phaseStageInfoPojos;
    }

    private List<LightsInfoPojo> getLightsInfo(CrossInfoPO crossInfo) {
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_LIGHT_CONFIG);
        String lightResultSign = Constants.COMMAND_LIGHT_CONFIG.concat(Constants.COMMAND_QUERY_REPLY);
        String lightsHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfo.getIp(), crossInfo.getPort(), lightsHex, lightResultSign, 300);
        return getLightsInfoPojos(crossInfo, resultPojo);
    }

    private static List<LightsInfoPojo> getLightsInfoPojos(CrossInfoPO crossInfo, MessageResultPojo resultPojo) {
        List<LightsInfoPojo> lightsInfoPojos = new ArrayList<>();
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int lightsSize = body.length() / 14;
            for (int i = 0; i < lightsSize; i++) {
                LightsInfoPojo lightsInfoPojo = new LightsInfoPojo();
                int lightId = HexUtil.hexToInt(body.substring(i * 14 + 6, i * 14 + 8));
                int type = HexUtil.hexToInt(body.substring(i * 14 + 8, i * 14 + 10));
                lightsInfoPojo.setLightId(lightId);
                lightsInfoPojo.setType(type);
                lightsInfoPojos.add(lightsInfoPojo);
            }
        }
        SignalDataCache.lightInfoCache.put(crossInfo.getId(), lightsInfoPojos);
        return lightsInfoPojos;
    }

    /**
     * 上载相位信息
     *
     * @param crossInfo
     */
    private List<PhaseInfoPojo> getPhaseInfo(CrossInfoPO crossInfo) {
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_PHASE_CONFIG);
        String phaseResultSign = Constants.COMMAND_PHASE_CONFIG.concat(Constants.COMMAND_QUERY_REPLY);
        String phaseHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfo.getIp(), crossInfo.getPort(), phaseHex, phaseResultSign, 300);
        Map<Integer, List<String>> phaseIdParamsMap = getHexPhaseParamMap(resultPojo);
        return getPhaseInfoPojos(crossInfo, phaseIdParamsMap);
    }

    private static List<PhaseInfoPojo> getPhaseInfoPojos(CrossInfoPO crossInfo, Map<Integer, List<String>> phaseIdParamsMap) {
        List<PhaseInfoPojo> phaseInfoPojos = new ArrayList<>();
        for (Map.Entry<Integer, List<String>> item : phaseIdParamsMap.entrySet()) {
            PhaseInfoPojo phaseInfoPojo = new PhaseInfoPojo();
            Integer phaseId = item.getKey();
            List<String> phaseParamList = item.getValue();
            if (!CollectionUtils.isEmpty(phaseParamList)) {
                String lightsIdHex = phaseParamList.get(0);
                StringBuffer sb = new StringBuffer();
                for (int i = 0; i < lightsIdHex.length() / 2; i++) {
                    String hex = lightsIdHex.substring(i * 2, i * 2 + 2);
                    String binary = Integer.toBinaryString(HexUtil.hexToInt(hex));
                    sb.append(StringUtils.leftPad(binary, 8, '0'));
                }
                phaseInfoPojo.setPhaseId(phaseId);
                phaseInfoPojo.setLightsId(sb.reverse().toString().indexOf("1") + 1);
                phaseInfoPojo.setGreenTime(HexUtil.hexToInt(phaseParamList.get(2)));
                phaseInfoPojo.setGreenFlashTime(HexUtil.hexToInt(phaseParamList.get(4)));
                phaseInfoPojo.setYellowTime(HexUtil.hexToInt(phaseParamList.get(6)));
                int redTime = HexUtil.hexToInt(phaseParamList.get(8)) - phaseInfoPojo.getGreenTime() - phaseInfoPojo.getGreenFlashTime() - phaseInfoPojo.getYellowTime();
                phaseInfoPojo.setRedTime(HexUtil.hexToInt(phaseParamList.get(8)));
                phaseInfoPojo.setYellowFlashTime(HexUtil.hexToInt(phaseParamList.get(14)));
                phaseInfoPojo.setMinGreenTime(HexUtil.hexToInt(phaseParamList.get(25)));
                phaseInfoPojo.setMaxGreenTime(HexUtil.hexToInt(phaseParamList.get(26)));
                phaseInfoPojo.setMaxGreenTime2(HexUtil.hexToInt(phaseParamList.get(27)));
                phaseInfoPojo.setExtendGreenTime(HexUtil.hexToInt(phaseParamList.get(28)));
                phaseInfoPojos.add(phaseInfoPojo);
            }
        }
        // 过滤掉未配置的相位
        List<PhaseInfoPojo> filter = phaseInfoPojos.stream().filter(phaseInfoPojo -> phaseInfoPojo.getRedTime() != 0).collect(Collectors.toList());
        SignalDataCache.phaseInfoCache.put(crossInfo.getId(), filter);
        return filter;
    }

    private static Map<Integer, List<String>> getHexPhaseParamMap(MessageResultPojo resultPojo) {
        Map<Integer, List<String>> phaseIdParamsMap = new HashMap<>();
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int phaseSize = body.length() / 456; // 456一组相位数据
            String matchStr = "0402%s%s";
            for (int i = 0; i < phaseSize - 1; i++) {
                int paramIndex = 0;
                int count = 1;
                List<String> paramList = new ArrayList<>();
                for (int j = 0; j < 31; j++) {
                    ++count;
                    // 0402 + 属性序号count + 相位序号i
                    String currentMatchStr = String.format(matchStr, String.format("%02x", count), String.format("%02x", i + 1));
                    paramIndex = body.indexOf(currentMatchStr) + 8;
                    String param = "";
                    if (count == 2) {
                        param = body.substring(paramIndex, paramIndex + 16);
                    } else if (count > 2 && count < 27) {
                        param = body.substring(paramIndex, paramIndex + 2);
                    } else if (count >= 27 && count < 31) {
                        param = body.substring(paramIndex, paramIndex + 4);
                    } else if (count == 31) {
                        param = body.substring(paramIndex, paramIndex + 16);
                    }
                    if (StringUtils.isBlank(param)) {
                        continue;
                    }
                    paramList.add(param);
                }
                phaseIdParamsMap.put(i + 1, paramList);
            }
        }
        return phaseIdParamsMap;
    }

    @Override
    public List<PlanSectionVO> planSection(PlanSectionDTO planSectionDTO) throws Exception {
        String crossId = planSectionDTO.getCrossId();
        CrossInfoPO crossInfoPO = DTControlCommandServiceImpl.checkCrossId(crossId);
        List<DailyPlanInfoPojo> dailyPlanInfoPojos = getDailyPlanInfo(crossInfoPO);
        return getPlanSectionVos(crossId, dailyPlanInfoPojos);
    }

    private static List<PlanSectionVO> getPlanSectionVos(String crossId, List<DailyPlanInfoPojo> dailyPlanInfoPojos) {
        List<PlanSectionVO> planSectionVOS = new ArrayList<>();
        for (DailyPlanInfoPojo dailyPlanInfoPojo : dailyPlanInfoPojos) {
            PlanSectionVO planSectionVO = new PlanSectionVO();
            Integer dailyPlanId = dailyPlanInfoPojo.getDailyPlanId();

            List<CrossPlanPO> crossPlanPOList = new ArrayList<>();
            CrossPlanPO crossPlanPO = new CrossPlanPO();
            crossPlanPO.setPlanNo(String.valueOf(dailyPlanId));
            crossPlanPO.setName(StringUtils.join("计划", dailyPlanId));
            crossPlanPO.setCrossId(crossId);
            crossPlanPOList.add(crossPlanPO);
            planSectionVO.setCrossPlanPOList(crossPlanPOList);

            List<String> startTimeChain = dailyPlanInfoPojo.getStartTimeChain();
            if (!CollectionUtils.isEmpty(startTimeChain)) {
                List<CrossSectionPO> crossSectionPOList = new ArrayList<>();
                for (int i = 0; i < startTimeChain.size(); i++) {
                    CrossSectionPO crossSectionPO = new CrossSectionPO();
                    crossSectionPO.setSectionNo(String.valueOf(i + 1));
                    crossSectionPO.setStartTime(startTimeChain.get(i));
                    if (i + 1 < startTimeChain.size()) {
                        crossSectionPO.setEndTime(startTimeChain.get(i + 1));
                    } else {
                        crossSectionPO.setEndTime(startTimeChain.get(0));
                    }
                    if (StringUtils.equals(crossSectionPO.getStartTime(), "23:59")) {
                        continue;
                    }
                    crossSectionPO.setCrossId(crossId);
                    crossSectionPO.setPlanId(dailyPlanId);
                    crossSectionPO.setSchemeId(dailyPlanInfoPojo.getSchemeIdChain().get(i));
                    crossSectionPO.setControlMode(ControlModelEnum.getWjControlMode(HexUtil.toHex(dailyPlanInfoPojo.getControlModelChain().get(i))));
                    crossSectionPOList.add(crossSectionPO);
                }
                planSectionVO.setCrossSectionPOList(crossSectionPOList);
                planSectionVOS.add(planSectionVO);
            }
        }
        return planSectionVOS;
    }

    private List<DailyPlanInfoPojo> getDailyPlanInfo(CrossInfoPO crossInfoPO) {
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_DAILYPLAN_CONFIG);
        String dailyPlanSign = Constants.COMMAND_DAILYPLAN_CONFIG.concat(Constants.COMMAND_QUERY_REPLY);
        String dailyPlanHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), dailyPlanHex, dailyPlanSign, 300);
        return getDailyPlanPojos(resultPojo);
    }

    private List<DailyPlanInfoPojo> getDailyPlanPojos(MessageResultPojo resultPojo) {
        List<DailyPlanInfoPojo> dailyPlanInfoPojos = new ArrayList<>();
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            HashMap<Integer, List<String>> paramMap = new HashMap<>();
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int dailyPlanSize = body.length() / 434;
            String matchStr = "0b02%s%s";
            for (int i = 0; i < dailyPlanSize; i++) {
                int paramIndex = 0;
                // 4个属性
                List<String> paramList = new ArrayList<>();
                for (int j = 2; j < 6; j++) {
                    String currentMatchStr = String.format(matchStr, String.format("%02x", j), String.format("%02x", i + 1));
                    paramIndex = body.indexOf(currentMatchStr) + 8;
                    String param = "";
                    if (j == 2) {
                        param = body.substring(paramIndex, paramIndex + 2);
                    } else if (j == 3) {
                        param = body.substring(paramIndex, paramIndex + 192);
                    } else {
                        param = body.substring(paramIndex, paramIndex + 96);
                    }
                    paramList.add(param);
                }
                paramMap.put(i + 1, paramList);
            }
            dailyPlanInfoPojos = convertDailyPlanPojos(paramMap);
        }
        return dailyPlanInfoPojos;
    }

    private List<DailyPlanInfoPojo> convertDailyPlanPojos(HashMap<Integer, List<String>> paramMap) {
        List<DailyPlanInfoPojo> dailyPlanInfoPojos = new ArrayList<>();
        for (Map.Entry<Integer, List<String>> entry : paramMap.entrySet()) {
            Integer dailyPlanId = entry.getKey();
            List<String> value = entry.getValue();
            if (!CollectionUtils.isEmpty(value)) {
                DailyPlanInfoPojo dailyPlanInfoPojo = new DailyPlanInfoPojo();
                dailyPlanInfoPojo.setDailyPlanId(dailyPlanId);
                dailyPlanInfoPojo.setControlCrossId(HexUtil.hexToInt(value.get(0)));
                List<Integer> listFromHex = getListFromHex(value.get(1), 2, "");
//                Collections.reverse(listFromHex);
                List<String> startTimeChain = new ArrayList<>();
                for (int i = 0; i < listFromHex.size() / 2; i++) {
                    int minute = listFromHex.get(i * 2);
                    int hour = listFromHex.get(i * 2 + 1);
                    // 只有第一个时段可以为00：00
                    if ((i != 0 && hour == 0)) {
                        continue;
                    }
                    startTimeChain.add(StringUtils.join(String.format("%02d", hour), ":", String.format("%02d", minute)));
                }
                dailyPlanInfoPojo.setStartTimeChain(startTimeChain);

                List<Integer> schemeIdChain = getListFromHex(value.get(2), 2, "00");
                dailyPlanInfoPojo.setSchemeIdChain(schemeIdChain);
                // 21 ==> 33 定周期
                List<Integer> controlModeChain = getListFromHex(value.get(3), 2, "00");
                dailyPlanInfoPojo.setControlModelChain(controlModeChain);
                dailyPlanInfoPojos.add(dailyPlanInfoPojo);
            }
        }
        return dailyPlanInfoPojos.stream().filter(dailyPlanInfoPojo -> !CollectionUtils.isEmpty(dailyPlanInfoPojo.getSchemeIdChain())).collect(Collectors.toList());
    }

    @Override
    public List<CrossSchedulesPO> crossSchedules(String crossId) throws Exception {
        CrossInfoPO crossInfoPO = DTControlCommandServiceImpl.checkCrossId(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_SCHEDULE_CONFIG);
        String scheduleSign = Constants.COMMAND_SCHEDULE_CONFIG.concat(Constants.COMMAND_QUERY_REPLY);
        String scheduleHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), scheduleHex, scheduleSign, 300);
        List<ScheduleInfoPojo> scheduleInfoPojos = getSchedulePojos(resultPojo);
        return getCrossSchedulesPOs(crossId, scheduleInfoPojos);
    }

    private List<CrossSchedulesPO> getCrossSchedulesPOs(String crossId, List<ScheduleInfoPojo> scheduleInfoPojos) throws Exception {
        List<CrossSchedulesPO> crossSchedulesPOS = new ArrayList<>(scheduleInfoPojos.size());
        for (ScheduleInfoPojo scheduleInfoPojo : scheduleInfoPojos) {
            List<Integer> week = scheduleInfoPojo.getWeek();
            for (Integer weekday : week) {
                CrossSchedulesPO crossSchedulesPO = new CrossSchedulesPO();
                crossSchedulesPO.setScheduleNo(scheduleInfoPojo.getScheduleId());
                crossSchedulesPO.setName(StringUtils.join("调度", scheduleInfoPojo.getScheduleId()));
                crossSchedulesPO.setCrossId(crossId);
                crossSchedulesPO.setPlanId(scheduleInfoPojo.getDailyPlanId());
                crossSchedulesPO.setWeek(weekday);
                crossSchedulesPOS.add(crossSchedulesPO);
            }
            if (week.size() != 7) {
                List<Integer> month = scheduleInfoPojo.getMonth();
                for (Integer currentMonth : month) {
                    LocalDate currentDate = LocalDate.now();
                    String currentYear = currentDate.toString().substring(0, 4);
                    for (Integer day : scheduleInfoPojo.getDay()) {
                        CrossSchedulesPO crossSchedulesPO = new CrossSchedulesPO();
                        crossSchedulesPO.setScheduleNo(scheduleInfoPojo.getScheduleId());
                        crossSchedulesPO.setName(StringUtils.join("调度", scheduleInfoPojo.getScheduleId()));
                        crossSchedulesPO.setCrossId(crossId);
                        crossSchedulesPO.setPlanId(scheduleInfoPojo.getDailyPlanId());
                        Date date = DateUtil.parse(StringUtils.join(currentYear, "-", currentMonth, "-", day), DateStyle.YYYY_MM_DD.getValue());
                        crossSchedulesPO.setSpecialDate(date);
                        crossSchedulesPOS.add(crossSchedulesPO);
                    }
                }
            }
        }
        return crossSchedulesPOS;
    }

    private List<ScheduleInfoPojo> getSchedulePojos(MessageResultPojo resultPojo) {
        List<ScheduleInfoPojo> scheduleInfoPojos = new ArrayList<>();
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            HashMap<Integer, List<String>> paramMap = new HashMap<>();
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int scheduleSize = body.length() / 92;
            String matchStr = "0c02%s%s";
            int paramIndex = 0;
            for (int i = 0; i < scheduleSize; i++) {
                List<String> paramList = new ArrayList<>();
                for (int j = 2; j < 8; j++) {
                    String currentMatchStr = String.format(matchStr, String.format("%02x", j), String.format("%02x", i + 1));
                    paramIndex = body.indexOf(currentMatchStr) + 8;
                    String param = "";
                    if (j == 5) {
                        param = body.substring(paramIndex, paramIndex + 4);
                    } else if (j == 6) {
                        param = body.substring(paramIndex, paramIndex + 8);
                    } else {
                        param = body.substring(paramIndex, paramIndex + 2);
                    }
                    paramList.add(param);
                }
                paramMap.put(i + 1, paramList);
            }
            scheduleInfoPojos = convertSchedulePojos(paramMap);

        }
        return scheduleInfoPojos;
    }

    private List<ScheduleInfoPojo> convertSchedulePojos(HashMap<Integer, List<String>> paramMap) {
        List<ScheduleInfoPojo> scheduleInfoPojos = new ArrayList<>();
        for (Map.Entry<Integer, List<String>> entry : paramMap.entrySet()) {
            Integer scheduleId = entry.getKey();
            List<String> value = entry.getValue();
            if (!CollectionUtils.isEmpty(value)) {
                ScheduleInfoPojo scheduleInfoPojo = new ScheduleInfoPojo();
                scheduleInfoPojo.setScheduleId(scheduleId);
                scheduleInfoPojo.setControlCrossId(HexUtil.hexToInt(value.get(0)));
                scheduleInfoPojo.setPriority(HexUtil.hexToInt(value.get(1)));
                String weekStr = Integer.toBinaryString(HexUtil.hexToInt(value.get(2)));
                List<Integer> weekList = new ArrayList<>();
                for (int i = 0; i < weekStr.length(); i++) {
                    String week = weekStr.substring(i, i + 1);
                    if (StringUtils.equals("1", week)) {
                        weekList.add(weekStr.length() - i);
                    }
                }
                scheduleInfoPojo.setWeek(weekList);

                String monthStr = Integer.toBinaryString(HexUtil.hexToInt(value.get(3)));
                List<Integer> monthList = new ArrayList<>();
                for (int i = 0; i < monthStr.length(); i++) {
                    String month = monthStr.substring(i, i + 1);
                    if (StringUtils.equals("1", month)) {
                        monthList.add(monthStr.length() - i);
                    }
                }
                scheduleInfoPojo.setMonth(monthList);

                String dayStr = Integer.toBinaryString(HexUtil.hexToInt(value.get(4)));
                List<Integer> dayList = new ArrayList<>();
                for (int i = 0; i < dayStr.length(); i++) {
                    String day = dayStr.substring(i, i + 1);
                    if (StringUtils.equals("1", day)) {
                        dayList.add(dayStr.length() - i);
                    }
                }
                scheduleInfoPojo.setDay(dayList);
                scheduleInfoPojo.setDailyPlanId(HexUtil.hexToInt(value.get(5)));
                scheduleInfoPojos.add(scheduleInfoPojo);
            }
        }
        List<ScheduleInfoPojo> filter = scheduleInfoPojos.stream().filter(scheduleInfoPojo -> Objects.nonNull(scheduleInfoPojo.getDailyPlanId())).collect(Collectors.toList());
        return filter;
    }

    @Override
    public Date signalTime(String crossId) throws Exception{
        CrossInfoPO crossInfoPO = DTControlCommandServiceImpl.checkCrossId(crossId);
        String message = String.format("010001000000010001%s010105%s0a0000", Constants.COMMAND_QUERY, Constants.COMMAND_DEVICE_STATUS_CONFIG);
        String signalTimeSign = Constants.COMMAND_DEVICE_STATUS_CONFIG.concat(Constants.COMMAND_QUERY_REPLY);
        String signalTimeHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), signalTimeHex, signalTimeSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            Date date = null;
            try {
                String hexMessageResult = resultPojo.getHexMessageResult();
                String yearHex = hexMessageResult.substring(40, 44);
                String monthHex = hexMessageResult.substring(44, 46);
                String dayHex = hexMessageResult.substring(46, 48);
                String hourHex = hexMessageResult.substring(48, 50);
                String minuteHex = hexMessageResult.substring(50, 52);
                String secondHex = hexMessageResult.substring(52, 54);
                String dateStr = String.join("-", String.valueOf(HexUtil.hexToInt(yearHex)), String.valueOf(HexUtil.hexToInt(monthHex)), String.valueOf(HexUtil.hexToInt(dayHex)));
                String time = String.join(":", String.valueOf(HexUtil.hexToInt(hourHex)), String.valueOf(HexUtil.hexToInt(minuteHex)), String.valueOf(HexUtil.hexToInt(secondHex)));
                date = DateUtil.parse((dateStr + " " + time), DateStyle.YYYY_MM_DD_HH_MM_SS.getValue());
            } catch (Exception e) {
                throw new Exception("信号机校时查询异常：", e);
            }
            return date;
        }
        return null;
    }

}
