package net.wanji.utc.dt.util;

import cn.hutool.core.util.HexUtil;
import freemarker.template.utility.StringUtil;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @author duanruiming
 * @date 2023/06/20 15:07
 */
@SuppressWarnings("all")
public class CommonUtils {

    /**
     * 将数组转化为16进制字符串
     *
     * @param array
     * @return
     */
    public static String getHex4Array(int[] array) {
        StringBuilder sb = new StringBuilder();
        for (int i : array) {
            sb.append(i);
        }
        Integer binary = Integer.valueOf(sb.reverse().toString(), 2);
        return HexUtil.toHex(binary);
    }

    /**
     * 将时间转化为16进制 00：00 --> 061E0
     *
     * @param time
     * @return
     */
    public static String getHourMinuteHex(String time) {
        StringBuffer sb = new StringBuffer();
        String[] split = time.split(":");
        String hour = String.format("%02x", Integer.valueOf(split[0]));
        String minute = String.format("%02x", Integer.valueOf(split[1]));
        return sb.append(hour).append(minute).toString();
    }

    /**
     * 将相位转化为64位数组，在转化为16位的16进制
     *
     * @param param
     * @return
     */
    public static String getBinary(Integer param) {
        int[] array = new int[64];
        array[param - 1] = 1;
        StringBuffer resultSb = new StringBuffer();
        for (int i = 0; i < array.length / 8; i++) {
            StringBuffer sb = new StringBuffer();
            for (int j = 0; j < 8; j++) {
                sb.append(array[i * 8 + j]);
            }
            String hex = String.format("%02x", Integer.parseInt(sb.reverse().toString(), 2));
            resultSb.insert(0, hex);
        }
        return StringUtil.leftPad(resultSb.toString(), 16, "0");
    }

    /**
     * 将相位列表转化为hex，每4个相位转化一位hex
     * @param phaseList
     * @return
     */
    public static String getBinary4PhaseList(List<Integer> phaseList) {
        int[] array = new int[64];
        if (!CollectionUtils.isEmpty(phaseList)) {
            for (Integer phaseId : phaseList) {
                array[phaseId - 1] = 1;
            }
        }
        StringBuffer resultSb = new StringBuffer();
        for (int i = 0; i < array.length / 4; i++) {
            StringBuffer sb = new StringBuffer();
            for (int j = 0; j < 4; j++) {
                sb.append(array[i * 4 + j]);
            }
            String hex = HexUtil.toHex(Integer.parseInt(sb.reverse().toString(), 2));
            resultSb.insert(0, hex);
        }
        return StringUtil.leftPad(resultSb.toString(), 16, "0");
    }

    /**
     * 将单个参数转化为4位二进制
     *
     * @param param
     * @return
     */
    public static Integer get4BinaryInteger(Integer param) {
        int[] array = new int[4];
        int index = param % 4;
        if (param % 4 != 0) {
            array[4 - index] = 1;
        } else {
            array[0] = 1;
        }
        StringBuffer sb = new StringBuffer();
        for (int j = 0; j < 4; j++) {
            sb.append(array[j]);
        }
        return Integer.parseInt(sb.toString(), 2);
    }

    /**
     * 通过灯组编号获取灯组转向， 1 左转 2 直右 3 非机动车 4 行人
     *
     * @param lightId
     * @return
     */
    public static Integer getTurnByLightsId(Integer lightId) {
        Integer turn = null;
        if (lightId % 4 == 0) {
            turn = 1;
        }
        if (lightId % 4 == 1) {
            turn = 2;
        }
        if (lightId % 4 == 2) {
            turn = 3;
        }
        if (lightId % 4 == 3) {
            turn = 20;
        }
        return turn;
    }

    public static Integer getDirByLightsId(Integer lightsId) {
        Integer dir = null;
        if (lightsId < 4) {
            dir = 1;
        }
        if (lightsId >= 4 && lightsId < 8) {
            dir = 3;
        }
        if (lightsId >= 8 && lightsId < 12) {
            dir = 5;
        }
        if (lightsId >= 12 && lightsId < 16) {
            dir = 7;
        }
        return dir;
    }

}
